using Azure.Identity;

using Microsoft.AspNetCore.Authentication.OpenIdConnect;
using Microsoft.AspNetCore.Http.Headers;
using Microsoft.Identity.Web;
using Microsoft.Identity.Web.UI;
using Microsoft.IdentityModel.Protocols.OpenIdConnect;

using NSeguin.Dev.Web;
using NSeguin.Dev.Web.Components;

WebApplicationBuilder builder = WebApplication.CreateBuilder(args);
builder.Logging.AddConfiguration(builder.Configuration.GetSection("Logging"));
builder.Logging.AddConsole();
builder.Services.Configure<CookiePolicyOptions>(
    options =>
    {
        // This lambda determines whether user consent for non-essential cookies is needed for a given request.
        options.CheckConsentNeeded = context => true;
        options.MinimumSameSitePolicy = SameSiteMode.Unspecified;

        // Handling SameSite cookie according to https://docs.microsoft.com/en-us/aspnet/core/security/samesite?view=aspnetcore-3.1
        options.HandleSameSiteCookieCompatibility();
    });

builder.Services.AddControllersWithViews().AddMicrosoftIdentityUI();
builder.Services.AddRazorPages();
builder.Services.AddRazorComponents()
    .AddInteractiveServerComponents()
    .AddInteractiveWebAssemblyComponents();

builder.Services.AddProblemDetails(
    options =>
    {
        options.CustomizeProblemDetails = ctx =>
        {
            ctx.ProblemDetails.Extensions.TryAdd("traceId", ctx.HttpContext.TraceIdentifier);
            if (ctx.HttpContext.RequestServices.GetRequiredService<IHostEnvironment>()
                .IsDevelopment())
            {
                ctx.ProblemDetails.Extensions.TryAdd("exception", ctx.Exception?.ToString());
            }
        };
    });

if (builder.Environment.IsStaging() || builder.Environment.IsProduction())
{
    builder.Configuration.AddAzureAppConfiguration(
        a =>
        {
            string? connectionString
                = Environment.GetEnvironmentVariable("AZURE_APP_CONFIGURATION_CONNECTION_STRING");

            if (string.IsNullOrWhiteSpace(connectionString))
            {
                throw new InvalidOperationException(
                    "The environment variable AZURE_APP_CONFIGURATION_CONNECTION_STRING is not set.");
            }

            a.Connect(connectionString)
                .ConfigureKeyVault(kv => kv.SetCredential(new EnvironmentCredential()));
        });
}
else
{
    builder.Configuration.AddUserSecrets<Program>();
}

builder.Services.AddAuthentication(OpenIdConnectDefaults.AuthenticationScheme)
    .AddMicrosoftIdentityWebApp(
        builder.Configuration.GetSection("AzureAdB2C"),
        subscribeToOpenIdConnectMiddlewareDiagnosticsEvents: true);

builder.Services.Configure<OpenIdConnectOptions>(
    OpenIdConnectDefaults.AuthenticationScheme,
    options =>
    {
        options.ResponseType = OpenIdConnectResponseType.Code;
        options.Scope.Add(options.ClientId!);
    });

builder.Services.AddHttpLogging(
    o =>
    {
        o.CombineLogs = true;
    });

WebApplication app = builder.Build();

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
    app.UseWebAssemblyDebugging();
}
else
{
    app.UseExceptionHandler("/Error", true);
}

app.UseHttpLogging();
app.UseHttpsRedirection();
app.UseBlazorFrameworkFiles();
app.UseStaticFiles(new StaticFileOptions { ServeUnknownFileTypes = true });
app.UseRouting();
app.UseAuthentication();
app.UseAuthorization();
app.UseAntiforgery();
app.MapRazorComponents<App>()
    .AddAdditionalAssemblies(typeof(AssemblyEntryPoint).Assembly)
    .AddInteractiveServerRenderMode()
    .AddInteractiveWebAssemblyRenderMode();

app.UseExceptionHandler();
app.MapFallback(
    ctx =>
    {
        ctx.Response.StatusCode = StatusCodes.Status404NotFound;
        RequestHeaders headers = ctx.Request.GetTypedHeaders();
        if (headers.Accept.Any(h => h.MatchesMediaType("text/html")))
        {
            ctx.Response.Redirect($"/NotFound?path={ctx.Request.Path}");
        }

        return Task.CompletedTask;
    });

app.Run();
